@@ -1,13 +1,13 @@
-/*-------------------------------------------------------------------------
+/* ----------------------------------------------------------------------
  * dbutils.c
  *
  *	Database utility functions for Slony-I
  *
- *	Copyright (c) 2003-2005, PostgreSQL Global Development Group
+ *	Copyright (c) 2003-2004, PostgreSQL Global Development Group
  *	Author: Jan Wieck, Afilias USA INC.
  *
- *	$Id: dbutils.c,v 1.16.2.1 2006-01-06 17:07:47 cbbrowne Exp $
- *-------------------------------------------------------------------------
+ *	$Id: dbutils.c,v 1.23 2006/07/26 12:32:09 darcyb Exp $
+ * ----------------------------------------------------------------------
  */
 
 
@@ -30,34 +30,44 @@
 
 
 static int	slon_appendquery_int(SlonDString * dsp, char *fmt, va_list ap);
+static int	db_get_version(PGconn *conn);
 
-/*
+#if (PG_VERSION_MAJOR < 8)
+/* ----
  * This mutex is used to wrap around PQconnectdb. There's a problem that
  * occurs when your libpq is compiled with libkrb (kerberos) which is not
  * threadsafe.	It is especially odd because I'm not using kerberos.
  *
  * This is fixed in libpq in 8.0, but for now (and for older versions we'll just
  * use this mutex.
- *
+ * ----
  */
 static pthread_mutex_t slon_connect_lock = PTHREAD_MUTEX_INITIALIZER;
+#endif
 
 
-/*
- * ---------- slon_connectdb ----------
+/* ----------
+ * slon_connectdb 
+ * ----------
  */
 SlonConn *
 slon_connectdb(char *conninfo, char *symname)
 {
 	PGconn	   *dbconn;
 	SlonConn   *conn;
+	PGresult   *res;
+	SlonDString query;
 
 	/*
 	 * Create the native database connection
 	 */
+#if (PG_VERSION_MAJOR < 8)
 	pthread_mutex_lock(&slon_connect_lock);
 	dbconn = PQconnectdb(conninfo);
 	pthread_mutex_unlock(&slon_connect_lock);
+#else
+	dbconn = PQconnectdb(conninfo);
+#endif
 	if (dbconn == NULL)
 	{
 		slon_log(SLON_ERROR,
@@ -73,38 +83,69 @@ slon_connectdb(char *conninfo, char *symname)
 		PQfinish(dbconn);
 		return NULL;
 	}
+
+	dstring_init(&query);
+
 	if (sql_on_connection != NULL)
 	{
 
-		PGresult *res;
-		SlonDString query;
-
-		dstring_init(&query);
 		slon_mkquery(&query, "%s", sql_on_connection);
 		res = PQexec(dbconn, dstring_data(&query));
-		if ( ! ((PQresultStatus(res) == PGRES_TUPLES_OK) ||
-		       (PQresultStatus(res)  == PGRES_COMMAND_OK)) )
+		if (!((PQresultStatus(res) == PGRES_TUPLES_OK) ||
+			  (PQresultStatus(res) == PGRES_COMMAND_OK)))
 		{
 			slon_log(SLON_ERROR,
-				"query %s failed\n",
-				dstring_data(&query));
+					 "query %s failed\n",
+					 dstring_data(&query));
 		}
-                PQclear(res);
+		PQclear(res);
 	}
 
+	/* set the datestyle to ISO */
+	slon_mkquery(&query, "set datestyle to 'ISO'");
+	res = PQexec(dbconn, dstring_data(&query));
+	if (!(PQresultStatus(res) == PGRES_COMMAND_OK))
+	{
+		slon_log(SLON_ERROR, "Unable to set the datestyle to ISO\n");
+	}
+	PQclear(res);
+	
 	/*
 	 * Embed it into a SlonConn structure used to exchange it with the
 	 * scheduler. On return this new connection object is locked.
 	 */
 	conn = slon_make_dummyconn(symname);
 	conn->dbconn = dbconn;
+	conn->pg_version = db_get_version(dbconn);
+        if (conn->pg_version < 70400)
+        {
+                slon_log(SLON_ERROR,
+                        "slon_connectdb: PQconnectdb(\"%s\") PostgreSQL version not supported\n",
+                        conninfo);
+                PQfinish(dbconn);
+                return NULL;
+        }
+	
+        slon_log(SLON_DEBUG4,
+                "version for \"%s\" is %d\n", conninfo, conn->pg_version);
 
+	if (conn->pg_version >= 80200)
+	{
+		slon_mkquery(&query, "set standard_conforming_strings to 'off'");
+		res = PQexec(dbconn, dstring_data(&query));
+		if (!(PQresultStatus(res) == PGRES_COMMAND_OK))
+		{
+			slon_log(SLON_ERROR, "Unable to set the standard_conforming_strings to off\n");
+        	}
+		PQclear(res);
+	}
 	return conn;
 }
 
 
-/*
- * ---------- slon_disconnectdb ----------
+/* ----------
+ * slon_disconnectdb 
+ * ----------
  */
 void
 slon_disconnectdb(SlonConn * conn)
@@ -124,8 +165,9 @@ slon_disconnectdb(SlonConn * conn)
 }
 
 
-/*
- * ---------- slon_make_dummyconn ----------
+/* ----------
+ * slon_make_dummyconn 
+ * ----------
  */
 SlonConn *
 slon_make_dummyconn(char *symname)
@@ -139,7 +181,7 @@ slon_make_dummyconn(char *symname)
 	if (conn == NULL)
 	{
 		perror("slon_make_dummyconn: malloc()");
-		slon_abort();
+		slon_retry();
 	}
 	memset(conn, 0, sizeof(SlonConn));
 	conn->symname = strdup(symname);
@@ -155,8 +197,9 @@ slon_make_dummyconn(char *symname)
 }
 
 
-/*
- * ---------- slon_free_dummyconn ----------
+/* ----------
+ * slon_free_dummyconn 
+ * ----------
  */
 void
 slon_free_dummyconn(SlonConn * conn)
@@ -179,15 +222,16 @@ slon_free_dummyconn(SlonConn * conn)
 }
 
 
-/*
- * ---------- db_getLocalNodeId
+/* ----------
+ * db_getLocalNodeId
  *
- * Query a connection for the value of sequence sl_local_node_id ----------
+ * Query a connection for the value of sequence sl_local_node_id 
+ * ----------
  */
 int
 db_getLocalNodeId(PGconn *conn)
 {
-	char		query  [1024];
+	char		query[1024];
 	PGresult   *res;
 	int			retval;
 
@@ -233,7 +277,7 @@ db_getLocalNodeId(PGconn *conn)
 int
 db_checkSchemaVersion(PGconn *conn)
 {
-	char		query  [1024];
+	char		query[1024];
 	PGresult   *res;
 	int			retval = 0;
 
@@ -323,13 +367,15 @@ db_checkSchemaVersion(PGconn *conn)
 }
 
 
-/*
- * ---------- slon_mkquery
+/* ----------
+ * slon_mkquery
  *
  * A simple query formatting and quoting function using dynamic string buffer
- * allocation. Similar to sprintf() it uses formatting symbols: %s
- * tring argument %q		Quoted literal (\ and ' will be escaped) %d
- * nteger argument ----------
+ * allocation. Similar to sprintf() it uses formatting symbols: 
+ *     %s	String argument
+ *     %q	Quoted literal (\ and ' will be escaped)
+ *     %d	Integer argument 
+ * ----------
  */
 int
 slon_mkquery(SlonDString * dsp, char *fmt,...)
@@ -348,10 +394,11 @@ slon_mkquery(SlonDString * dsp, char *fmt,...)
 }
 
 
-/*
- * ---------- slon_appendquery
+/* ----------
+ * slon_appendquery
  *
- * Append query string material to an existing dynamic string. ----------
+ * Append query string material to an existing dynamic string.
+ * ----------
  */
 int
 slon_appendquery(SlonDString * dsp, char *fmt,...)
@@ -368,16 +415,17 @@ slon_appendquery(SlonDString * dsp, char *fmt,...)
 }
 
 
-/*
- * ---------- slon_appendquery_int
+/* ----------
+ * slon_appendquery_int
  *
- * Implementation of slon_mkquery() and slon_appendquery(). ----------
+ * Implementation of slon_mkquery() and slon_appendquery(). 
+ * ----------
  */
 static int
 slon_appendquery_int(SlonDString * dsp, char *fmt, va_list ap)
 {
 	char	   *s;
-	char		buf    [64];
+	char		buf[64];
 
 	while (*fmt)
 	{
@@ -446,6 +494,37 @@ slon_appendquery_int(SlonDString * dsp, char *fmt, va_list ap)
 	return 0;
 }
 
+static int db_get_version(PGconn *conn)
+{
+	PGresult    *res;
+	SlonDString query;
+	char	    versionstr[7];
+	int	    version=0;
+	int	    major=0;
+	int	    minor=0;
+	int	    patch=0;
+
+	dstring_init(&query);
+	slon_mkquery(&query, "SELECT version();");
+	res = PQexec(conn, dstring_data(&query));
+
+	if ( !res || PQresultStatus(res) != PGRES_TUPLES_OK )
+        {
+		PQclear(res);
+		return -1;
+	}
+	if (sscanf(PQgetvalue(res, 0, 0), "PostgreSQL %d.%d.%d", &major, &minor, &patch) < 2)
+	{
+		PQclear(res);
+		return -1;
+	}
+	PQclear(res);
+	snprintf(versionstr, 7, "%.2d%.2d%.2d", major, minor, patch);
+	version=atoi(versionstr);
+
+	return version;
+}
+	
 /*
  * Local Variables:
  *	tab-width: 4
