@@ -3,10 +3,10 @@
  *
  *	Miscellaneous support functions
  *
- *	Copyright (c) 2003-2006, PostgreSQL Global Development Group
+ *	Copyright (c) 2003-2004, PostgreSQL Global Development Group
  *	Author: Jan Wieck, Afilias USA INC.
  *
- *	$Id: misc.c,v 1.18.2.1 2006-01-06 17:07:47 cbbrowne Exp $
+ *	$Id: misc.c,v 1.22 2005/11/22 05:11:58 wieck Exp $
  *-------------------------------------------------------------------------
  */
 
@@ -24,7 +24,11 @@
 #include <sys/time.h>
 #include <sys/types.h>
 
+#ifndef WIN32
 #include <syslog.h>
+#else
+#include "port/win32service.h"
+#endif
 #include <stdarg.h>
 
 #include "libpq-fe.h"
@@ -59,13 +63,16 @@ extern char *Syslog_facility;	/* openlog() parameters */
 extern char *Syslog_ident;
 
 static void write_syslog(int level, const char *line);
-
 #else
 
 #define Use_syslog 0
 #endif   /* HAVE_SYSLOG */
 
 
+/* ----------
+ * slon_log
+ * ----------
+ */
 void
 slon_log(Slon_Log_Level level, char *fmt,...)
 {
@@ -142,11 +149,16 @@ slon_log(Slon_Log_Level level, char *fmt,...)
 		{
 			perror("slon_log: malloc()");
 			pthread_mutex_unlock(&log_mutex);
-			slon_abort();
+			slon_retry();
 		}
 	}
+	outbuf[0] = 0;
 
-	if (logtimestamp == true && (Use_syslog != 1))
+	if (logtimestamp == true && (Use_syslog != 1)
+#ifdef WIN32
+		&& !win32_isservice
+#endif
+		)
 	{
 		strftime(time_buf, sizeof(time_buf), log_timestamp_format, localtime(&stamp_time));
 		sprintf(outbuf, "%s ", time_buf);
@@ -166,7 +178,7 @@ slon_log(Slon_Log_Level level, char *fmt,...)
 		if (outbuf == NULL)
 		{
 			perror("slon_log: realloc()");
-			slon_abort();
+			slon_retry();
 		}
 	}
 #ifdef HAVE_SYSLOG
@@ -174,6 +186,10 @@ slon_log(Slon_Log_Level level, char *fmt,...)
 	{
 		write_syslog(syslog_level, outbuf);
 	}
+#endif
+#ifdef WIN32
+	if (win32_isservice)
+		win32_eventlog(level, outbuf);
 #endif
 	fwrite(outbuf, strlen(outbuf), 1, stdout);
 	fflush(stdout);
@@ -183,11 +199,12 @@ slon_log(Slon_Log_Level level, char *fmt,...)
 }
 
 
-/*
+/* ----------
  * scanint8 --- try to parse a string into an int8.
  *
  * If errorOK is false, ereport a useful error message if the string is bad. If
  * errorOK is true, just return "false" for bad input.
+ * ----------
  */
 int
 slon_scanint64(char *str, int64 * result)
@@ -250,6 +267,10 @@ slon_scanint64(char *str, int64 * result)
 }
 
 #if HAVE_SYSLOG
+/* ----------
+ * write_syslog
+ * ----------
+ */
 static void
 write_syslog(int level, const char *line)
 {
@@ -298,7 +319,7 @@ write_syslog(int level, const char *line)
 
 		while (len > 0)
 		{
-			char		buf    [SLON_SYSLOG_LIMIT + 1];
+			char		buf[SLON_SYSLOG_LIMIT + 1];
 			int			buflen;
 			int			i;
 
@@ -352,3 +373,5 @@ write_syslog(int level, const char *line)
 }
 
 #endif   /* HAVE_SYSLOG */
+
+
