@@ -3,10 +3,10 @@
  *
  *	Functions maintaining the in-memory configuration information
  *
- *	Copyright (c) 2003-2006, PostgreSQL Global Development Group
+ *	Copyright (c) 2003-2004, PostgreSQL Global Development Group
  *	Author: Jan Wieck, Afilias USA INC.
  *
- *	$Id: runtime_config.c,v 1.24.2.1 2006-01-06 17:07:47 cbbrowne Exp $
+ *	$Id: runtime_config.c,v 1.27 2005/11/22 05:11:59 wieck Exp $
  *-------------------------------------------------------------------------
  */
 
@@ -29,12 +29,17 @@
 #include "slon.h"
 
 
-/*
- * ---------- Global data ----------
+/* ----------
+ * Global data 
+ * ----------
  */
 pid_t		slon_pid;
-pid_t		slon_cpid;
-pid_t		slon_ppid;
+
+#ifndef WIN32
+pthread_mutex_t slon_watchdog_lock;
+pid_t		slon_watchdog_pid;
+pid_t		slon_worker_pid;
+#endif
 char	   *rtcfg_cluster_name = NULL;
 char	   *rtcfg_namespace = NULL;
 char	   *rtcfg_conninfo = NULL;
@@ -49,8 +54,9 @@ SlonNode   *rtcfg_node_list_head = NULL;
 SlonNode   *rtcfg_node_list_tail = NULL;
 
 
-/*
- * ---------- Local data ----------
+/* ----------
+ * Local data 
+ * ----------
  */
 static pthread_mutex_t config_lock = PTHREAD_MUTEX_INITIALIZER;
 static pthread_mutex_t cfgseq_lock = PTHREAD_MUTEX_INITIALIZER;
@@ -67,14 +73,16 @@ static struct to_activate *to_activate_head = NULL;
 static struct to_activate *to_activate_tail = NULL;
 
 
-/*
- * ---------- Local functions ----------
+/* ----------
+ * Local functions 
+ * ----------
  */
 static void rtcfg_startStopNodeThread(SlonNode * node);
 
 
-/*
- * ---------- rtcfg_lock ----------
+/* ----------
+ * rtcfg_lock 
+ * ----------
  */
 void
 rtcfg_lock(void)
@@ -83,8 +91,9 @@ rtcfg_lock(void)
 }
 
 
-/*
- * ---------- rtcfg_unlock ----------
+/* ----------
+ * rtcfg_unlock 
+ * ----------
  */
 void
 rtcfg_unlock(void)
@@ -93,8 +102,9 @@ rtcfg_unlock(void)
 }
 
 
-/*
- * ---------- rtcfg_storeNode ----------
+/* ----------
+ * rtcfg_storeNode 
+ * ----------
  */
 void
 rtcfg_storeNode(int no_id, char *no_comment)
@@ -131,7 +141,7 @@ rtcfg_storeNode(int no_id, char *no_comment)
 	if (node == NULL)
 	{
 		perror("rtcfg_storeNode: malloc()");
-		slon_abort();
+		slon_retry();
 	}
 	memset(node, 0, sizeof(SlonNode));
 
@@ -148,13 +158,14 @@ rtcfg_storeNode(int no_id, char *no_comment)
 }
 
 
-/*
- * ---------- rtcfg_setNodeLastEvent()
+/* ----------
+ * rtcfg_setNodeLastEvent()
  *
  * Set the last_event field in the node runtime structure.
  *
  * Returns: 0 if the event_seq is <= the known value -1 if the node is
- * not known event_seq otherwise ----------
+ * not known event_seq otherwise 
+ * ----------
  */
 int64
 rtcfg_setNodeLastEvent(int no_id, int64 event_seq)
@@ -186,10 +197,11 @@ rtcfg_setNodeLastEvent(int no_id, int64 event_seq)
 }
 
 
-/*
- * ---------- rtcfg_getNodeLastEvent
+/* ----------
+ * rtcfg_getNodeLastEvent
  *
- * Read the nodes last_event field ----------
+ * Read the nodes last_event field 
+ * ----------
  */
 int64
 rtcfg_getNodeLastEvent(int no_id)
@@ -210,9 +222,10 @@ rtcfg_getNodeLastEvent(int no_id)
 	return retval;
 }
 
-
-/*
- * ---------- rtcfg_enableNode ----------
+ 
+/* ---------- 
+ * rtcfg_enableNode 
+ * ----------
  */
 void
 rtcfg_enableNode(int no_id)
@@ -228,7 +241,7 @@ rtcfg_enableNode(int no_id)
 
 		slon_log(SLON_FATAL,
 				 "enableNode: unknown node ID %d\n", no_id);
-		slon_abort();
+		slon_retry();
 		return;
 	}
 
@@ -246,8 +259,9 @@ rtcfg_enableNode(int no_id)
 }
 
 
-/*
- * ---------- slon_disableNode ----------
+/* ---------- 
+ * slon_disableNode 
+ * ----------
  */
 void
 rtcfg_disableNode(int no_id)
@@ -263,7 +277,7 @@ rtcfg_disableNode(int no_id)
 
 		slon_log(SLON_FATAL,
 				 "enableNode: unknown node ID %d\n", no_id);
-		slon_abort();
+		slon_retry();
 		return;
 	}
 
@@ -283,8 +297,9 @@ rtcfg_disableNode(int no_id)
 }
 
 
-/*
- * ---------- rtcfg_findNode ----------
+/* ---------- 
+ * rtcfg_findNode 
+ * ----------
  */
 SlonNode *
 rtcfg_findNode(int no_id)
@@ -301,8 +316,9 @@ rtcfg_findNode(int no_id)
 }
 
 
-/*
- * ---------- rtcfg_storePath ----------
+/* ---------- 
+ * rtcfg_storePath 
+ * ----------
  */
 void
 rtcfg_storePath(int pa_server, char *pa_conninfo, int pa_connretry)
@@ -347,8 +363,9 @@ rtcfg_storePath(int pa_server, char *pa_conninfo, int pa_connretry)
 }
 
 
-/*
- * ---------- rtcfg_dropPath ----------
+/* ---------- 
+ * rtcfg_dropPath 
+ * ----------
  */
 void
 rtcfg_dropPath(int pa_server)
@@ -403,15 +420,16 @@ rtcfg_dropPath(int pa_server)
 }
 
 
-/*
- * ---------- rtcfg_storeListen ----------
+/* ---------- 
+ * rtcfg_storeListen 
+ * ----------
  */
 void
 rtcfg_reloadListen(PGconn *db)
 {
 	SlonDString query;
 	PGresult   *res;
-	int			i	  ,
+	int			i,
 				n;
 	SlonNode   *node;
 	SlonListen *listen;
@@ -447,7 +465,7 @@ rtcfg_reloadListen(PGconn *db)
 				 PQresultErrorMessage(res));
 		PQclear(res);
 		dstring_free(&query);
-		slon_exit(-1);
+		slon_retry();
 	}
 	for (i = 0, n = PQntuples(res); i < n; i++)
 	{
@@ -467,8 +485,9 @@ rtcfg_reloadListen(PGconn *db)
 }
 
 
-/*
- * ---------- rtcfg_storeListen ----------
+/* ---------- 
+ * rtcfg_storeListen 
+ * ----------
  */
 void
 rtcfg_storeListen(int li_origin, int li_provider)
@@ -483,7 +502,7 @@ rtcfg_storeListen(int li_origin, int li_provider)
 	{
 		slon_log(SLON_FATAL,
 				 "storeListen: unknown node ID %d\n", li_provider);
-		slon_abort();
+		slon_retry();
 		return;
 	}
 
@@ -515,7 +534,7 @@ rtcfg_storeListen(int li_origin, int li_provider)
 	if (listen == NULL)
 	{
 		perror("rtcfg_storeListen: malloc()");
-		slon_abort();
+		slon_retry();
 	}
 	memset(listen, 0, sizeof(SlonListen));
 
@@ -532,8 +551,9 @@ rtcfg_storeListen(int li_origin, int li_provider)
 }
 
 
-/*
- * ---------- rtcfg_dropListen ----------
+/* ---------- 
+ * rtcfg_dropListen 
+ * ----------
  */
 void
 rtcfg_dropListen(int li_origin, int li_provider)
@@ -548,7 +568,7 @@ rtcfg_dropListen(int li_origin, int li_provider)
 	{
 		slon_log(SLON_FATAL,
 				 "dropListen: unknown node ID %d\n", li_provider);
-		slon_abort();
+		slon_retry();
 		return;
 	}
 
@@ -590,6 +610,10 @@ rtcfg_dropListen(int li_origin, int li_provider)
 }
 
 
+/* ---------- 
+ * rtcfg_storeSet 
+ * ----------
+ */
 void
 rtcfg_storeSet(int set_id, int set_origin, char *set_comment)
 {
@@ -636,7 +660,7 @@ rtcfg_storeSet(int set_id, int set_origin, char *set_comment)
 	if (set == NULL)
 	{
 		perror("rtcfg_storeSet: malloc()");
-		slon_abort();
+		slon_retry();
 	}
 	memset(set, 0, sizeof(SlonSet));
 
@@ -653,6 +677,10 @@ rtcfg_storeSet(int set_id, int set_origin, char *set_comment)
 }
 
 
+/* ---------- 
+ * rtcfg_dropSet 
+ * ----------
+ */
 void
 rtcfg_dropSet(int set_id)
 {
@@ -687,6 +715,10 @@ rtcfg_dropSet(int set_id)
 	rtcfg_unlock();
 }
 
+/* ---------- 
+ * rtcfg_moveSet 
+ * ----------
+ */
 void
 rtcfg_moveSet(int set_id, int old_origin, int new_origin, int sub_provider)
 {
@@ -731,10 +763,14 @@ rtcfg_moveSet(int set_id, int old_origin, int new_origin, int sub_provider)
 	 */
 	rtcfg_unlock();
 	slon_log(SLON_FATAL, "rtcfg_moveSet(): set %d not found\n", set_id);
-	slon_abort();
+	slon_retry();
 }
 
 
+/* ---------- 
+ * rtcfg_storeSubscribe 
+ * ----------
+ */
 void
 rtcfg_storeSubscribe(int sub_set, int sub_provider, char *sub_forward)
 {
@@ -776,10 +812,14 @@ rtcfg_storeSubscribe(int sub_set, int sub_provider, char *sub_forward)
 	slon_log(SLON_FATAL,
 			 "storeSubscribe: set %d not found\n", sub_set);
 	rtcfg_unlock();
-	slon_abort();
+	slon_retry();
 }
 
 
+/* ---------- 
+ * rtcfg_enableSubscription 
+ * ----------
+ */
 void
 rtcfg_enableSubscription(int sub_set, int sub_provider, char *sub_forward)
 {
@@ -818,10 +858,14 @@ rtcfg_enableSubscription(int sub_set, int sub_provider, char *sub_forward)
 	slon_log(SLON_FATAL,
 			 "enableSubscription: set %d not found\n", sub_set);
 	rtcfg_unlock();
-	slon_abort();
+	slon_retry();
 }
 
 
+/* ---------- 
+ * rtcfg_unsubscribeSet 
+ * ----------
+ */
 void
 rtcfg_unsubscribeSet(int sub_set)
 {
@@ -859,12 +903,13 @@ rtcfg_unsubscribeSet(int sub_set)
 	slon_log(SLON_FATAL,
 			 "unsubscribeSet: set %d not found\n", sub_set);
 	rtcfg_unlock();
-	slon_abort();
+	slon_retry();
 }
 
 
-/*
- * ---------- rtcfg_startStopNodeThread ----------
+/* ---------- 
+ * rtcfg_startStopNodeThread 
+ * ----------
  */
 static void
 rtcfg_startStopNodeThread(SlonNode * node)
@@ -890,7 +935,7 @@ rtcfg_startStopNodeThread(SlonNode * node)
 							 "remoteWorkerThread - %s\n",
 							 strerror(errno));
 					rtcfg_unlock();
-					slon_abort();
+					slon_retry();
 				}
 				node->worker_status = SLON_TSTAT_RUNNING;
 				break;
@@ -944,7 +989,7 @@ rtcfg_startStopNodeThread(SlonNode * node)
 							 "remoteListenThread - %s\n",
 							 strerror(errno));
 					rtcfg_unlock();
-					slon_abort();
+					slon_retry();
 				}
 				break;
 
@@ -1008,22 +1053,30 @@ rtcfg_startStopNodeThread(SlonNode * node)
 }
 
 
+/* ---------- 
+ * rtcfg_needActivate 
+ * ----------
+ */
 void
 rtcfg_needActivate(int no_id)
 {
 	struct to_activate *anode;
 
-	anode = (struct to_activate *)malloc(sizeof(struct to_activate));
+	anode = (struct to_activate *) malloc(sizeof(struct to_activate));
 	if (anode == NULL)
 	{
 		perror("rtcfg_needActivate: malloc()");
-		slon_abort();
+		slon_retry();
 	}
 	anode->no_id = no_id;
 	DLLIST_ADD_TAIL(to_activate_head, to_activate_tail, anode);
 }
 
 
+/* ---------- 
+ * rtcfg_doActivate 
+ * ----------
+ */
 void
 rtcfg_doActivate(void)
 {
@@ -1038,6 +1091,10 @@ rtcfg_doActivate(void)
 }
 
 
+/* ---------- 
+ * rtcfg_joinAllRemoteThreads 
+ * ----------
+ */
 void
 rtcfg_joinAllRemoteThreads(void)
 {
@@ -1093,6 +1150,10 @@ rtcfg_joinAllRemoteThreads(void)
 }
 
 
+/* ---------- 
+ * rtcfg_seq_bump 
+ * ----------
+ */
 void
 rtcfg_seq_bump(void)
 {
@@ -1102,6 +1163,10 @@ rtcfg_seq_bump(void)
 }
 
 
+/* ---------- 
+ * rtcfg_seq_get 
+ * ----------
+ */
 int64
 rtcfg_seq_get(void)
 {
