@@ -1,13 +1,13 @@
-/*-------------------------------------------------------------------------
+/* ----------------------------------------------------------------------
  * scheduler.c
  *
  *	Event scheduling subsystem for slon.
  *
- *	Copyright (c) 2003-2006, PostgreSQL Global Development Group
+ *	Copyright (c) 2003-2004, PostgreSQL Global Development Group
  *	Author: Jan Wieck, Afilias USA INC.
  *
- *	$Id: scheduler.c,v 1.20.2.1 2006-01-06 17:07:47 cbbrowne Exp $
- *-------------------------------------------------------------------------
+ *	$Id: scheduler.c,v 1.24 2006/08/04 21:01:18 cbbrowne Exp $
+ * ----------------------------------------------------------------------
  */
 
 
@@ -36,8 +36,9 @@
 #define PF_LOCAL PF_UNIX
 #endif
 
-/*
- * ---------- Static data ----------
+/* ---------- 
+ * Static data 
+ * ----------
  */
 static int	sched_status = SCHED_STATUS_OK;
 
@@ -50,41 +51,67 @@ static SlonConn *sched_waitqueue_tail = NULL;
 static pthread_t sched_main_thread;
 static pthread_t sched_scheduler_thread;
 
-static pthread_mutex_t sched_master_lock = PTHREAD_MUTEX_INITIALIZER;
-static pthread_cond_t sched_master_cond = PTHREAD_COND_INITIALIZER;
+static pthread_mutex_t sched_master_lock;
+static pthread_cond_t sched_master_cond;
 
 
-/*
- * ---------- Local functions ----------
+/* ----------
+ * Local functions 
+ * ----------
  */
 static void *sched_mainloop(void *);
 static void sched_add_fdset(int fd, fd_set * fds);
 static void sched_remove_fdset(int fd, fd_set * fds);
-static void sched_shutdown();
+/* static void sched_shutdown(); */
 
 
-/*
- * ---------- sched_start_mainloop
+/* ----------
+ * sched_start_mainloop
  *
- * Called from main() before starting up any worker thread.
+ * Called from SlonMain() before starting up any worker thread.
  *
  * This will spawn the event scheduling thread that does the central select(2)
- * system call. ----------
+ * system call. 
+ * ----------
  */
 int
 sched_start_mainloop(void)
 {
+	sched_status = SCHED_STATUS_OK;
+	sched_waitqueue_head = NULL;
+	sched_waitqueue_tail = NULL;
+	sched_numfd = 0;
+	FD_ZERO(&sched_fdset_read);
+	FD_ZERO(&sched_fdset_write);
+
 	/*
 	 * Remember the main threads identifier
 	 */
 	sched_main_thread = pthread_self();
 
+	/*
+	 * Initialize the master lock and condition variables
+	 */
+	if (pthread_mutex_init(&sched_master_lock, NULL) < 0)
+	{
+		slon_log(SLON_FATAL, "sched_start_mainloop: pthread_mutex_init() - %s\n",
+				 strerror(errno));
+		return -1;
+	}
+	if (pthread_cond_init(&sched_master_cond, NULL) < 0)
+	{
+		slon_log(SLON_FATAL, "sched_start_mainloop: pthread_cond_init() - %s\n",
+				 strerror(errno));
+		return -1;
+	}
+
 	/*
 	 * Grab the scheduler master lock
 	 */
 	if (pthread_mutex_lock(&sched_master_lock) < 0)
 	{
-		perror("sched_start_mainloop: pthread_mutex_lock()");
+		slon_log(SLON_FATAL, "sched_start_mainloop: pthread_mutex_lock() - %s\n",
+				 strerror(errno));
 		return -1;
 	}
 
@@ -93,7 +120,8 @@ sched_start_mainloop(void)
 	 */
 	if (pthread_create(&sched_scheduler_thread, NULL, sched_mainloop, NULL) < 0)
 	{
-		perror("sched_start_mainloop: pthread_create()");
+		slon_log(SLON_FATAL, "sched_start_mainloop: pthread_create() - %s\n",
+				 strerror(errno));
 		return -1;
 	}
 
@@ -102,7 +130,8 @@ sched_start_mainloop(void)
 	 */
 	if (pthread_cond_wait(&sched_master_cond, &sched_master_lock) < 0)
 	{
-		perror("sched_start_mainloop: pthread_cond_wait()");
+		slon_log(SLON_FATAL, "sched_start_mainloop: pthread_cond_wait() - %s\n",
+				 strerror(errno));
 		return -1;
 	}
 
@@ -111,7 +140,8 @@ sched_start_mainloop(void)
 	 */
 	if (pthread_mutex_unlock(&sched_master_lock) < 0)
 	{
-		perror("sched_start_mainloop: pthread_mutex_unlock()");
+		slon_log(SLON_FATAL, "sched_start_mainloop: pthread_mutex_unlock() - %s\n",
+				 strerror(errno));
 		return -1;
 	}
 
@@ -128,12 +158,13 @@ sched_start_mainloop(void)
 }
 
 
-/*
- * ---------- sched_wait_mainloop
+/* ----------
+ * sched_wait_mainloop
  *
  * Called from main() after all working threads according to the initial
  * configuration are started. Will wait until the scheduler mainloop
- * terminates. ----------
+ * terminates. 
+ * ----------
  */
 int
 sched_wait_mainloop(void)
@@ -150,13 +181,14 @@ sched_wait_mainloop(void)
 }
 
 
-/*
- * ---------- sched_wait_conn
+/* ----------
+ * sched_wait_conn
  *
  * Assumes that the thread holds the lock on conn->conn_lock.
  *
  * Adds the connection to the central wait queue and wakes up the scheduler
- * thread to reloop onto the select(2) call. ----------
+ * thread to reloop onto the select(2) call. 
+ * ----------
  */
 int
 sched_wait_conn(SlonConn * conn, int condition)
@@ -192,7 +224,7 @@ sched_wait_conn(SlonConn * conn, int condition)
 	 * Give the scheduler thread a heads up, release the master lock and wait
 	 * for it to tell us that the event we're waiting for happened.
 	 */
-	if (write(sched_wakeuppipe[1], "x", 1) < 0)
+	if (pipewrite(sched_wakeuppipe[1], "x", 1) < 0)
 	{
 		perror("sched_wait_conn: write()");
 		exit(-1);
@@ -218,8 +250,8 @@ sched_wait_conn(SlonConn * conn, int condition)
 }
 
 
-/*
- * ---------- sched_wait_time
+/* ---------- 
+ * sched_wait_time
  *
  * Assumes that the thread holds the lock on conn->conn_lock.
  *
@@ -247,10 +279,11 @@ sched_wait_time(SlonConn * conn, int condition, int msec)
 }
 
 
-/*
- * ---------- sched_msleep
+/* ---------- 
+ * sched_msleep
  *
- * Use the schedulers event loop to sleep for msec milliseconds. ----------
+ * Use the schedulers event loop to sleep for msec milliseconds. 
+ * ----------
  */
 int
 sched_msleep(SlonNode * node, int msec)
@@ -274,10 +307,11 @@ sched_msleep(SlonNode * node, int msec)
 }
 
 
-/*
- * ---------- sched_get_status
+/* ---------- 
+ * sched_get_status
  *
- * Return the current scheduler status in a thread safe fashion ----------
+ * Return the current scheduler status in a thread safe fashion 
+ * ----------
  */
 int
 sched_get_status(void)
@@ -291,12 +325,13 @@ sched_get_status(void)
 }
 
 
-/*
- * ---------- sched_wakeup_node
+/* ---------- 
+ * sched_wakeup_node
  *
  * Wakeup the threads (listen and worker) of one or all remote nodes to cause
  * them rechecking the current runtime status or adjust their configuration
- * to changes. ----------
+ * to changes. 
+ * ----------
  */
 int
 sched_wakeup_node(int no_id)
@@ -326,10 +361,10 @@ sched_wakeup_node(int no_id)
 	 */
 	if (num_wakeup > 0)
 	{
-		if (write(sched_wakeuppipe[1], "x", 1) < 0)
+		if (pipewrite(sched_wakeuppipe[1], "x", 1) < 0)
 		{
 			perror("sched_wait_conn: write()");
-			slon_abort();
+			slon_restart();
 		}
 	}
 	pthread_mutex_unlock(&sched_master_lock);
@@ -343,10 +378,11 @@ sched_wakeup_node(int no_id)
 }
 
 
-/*
- * ---------- sched_mainloop
+/* ---------- 
+ * sched_mainloop
  *
- * The thread handling the master scheduling. ----------
+ * The thread handling the master scheduling. 
+ * ----------
  */
 static void *
 sched_mainloop(void *dummy)
@@ -514,10 +550,10 @@ sched_mainloop(void *dummy)
 		 */
 		if (FD_ISSET(sched_wakeuppipe[0], &rfds))
 		{
-			char		buf    [1];
+			char		buf[1];
 
 			rc--;
-			if (read(sched_wakeuppipe[0], buf, 1) != 1)
+			if (piperead(sched_wakeuppipe[0], buf, 1) != 1)
 			{
 				perror("sched_mainloop: read()");
 				sched_status = SCHED_STATUS_ERROR;
@@ -599,11 +635,9 @@ sched_mainloop(void *dummy)
 	 */
 
 	/*
-	close(sched_wakeuppipe[0]);
-	sched_wakeuppipe[0] = -1;
-	close(sched_wakeuppipe[1]);
-	sched_wakeuppipe[1] = -1;
-	*/
+	 * close(sched_wakeuppipe[0]); sched_wakeuppipe[0] = -1;
+	 * close(sched_wakeuppipe[1]); sched_wakeuppipe[1] = -1;
+	 */
 
 	/*
 	 * Then we cond_signal all connections that are in the queue.
@@ -637,65 +671,12 @@ sched_mainloop(void *dummy)
 }
 
 
-/*
- * ---------- sched_sighandler
- *
- * After starting up the sole purpose of the main thread (the original process)
- * is to respond to signals while waiting for the scheduler to finish. The
- * signal handler is here because it is actually sched_start_mainloop() that
- * arranges for the signal handling and sched_wait_mainloop() that enables
- * them. And the only one really interested in the signals is the scheduler
- * thread ... but that's doing select(2) mainly and we have to avoid race
- * conditions with signals. ----------
- */
-static void
-sched_shutdown()
-{
-	/*
-	 * Lock the master mutex and make sure that we are the main thread
-	 */
-	pthread_mutex_lock(&sched_master_lock);
-	if (!pthread_equal(pthread_self(), sched_main_thread))
-	{
-		slon_log(SLON_FATAL, "sched_sighandler: called in non-main thread\n");
-		slon_abort();
-	}
-
-	/*
-	 * Set the scheduling status to shutdown
-	 */
-	if (sched_status == SCHED_STATUS_OK)
-		sched_status = SCHED_STATUS_SHUTDOWN;
-
-	/*
-	 * Try to wakeup the scheduler thread by throwing a bait
-	 */
-	if (sched_wakeuppipe[1] < 0)
-	{
-		slon_log(SLON_ERROR, "sched_sighandler: sockpair already closed\n");
-		pthread_mutex_unlock(&sched_master_lock);
-		return;
-	}
-	if (write(sched_wakeuppipe[1], "x", 1) < 0)
-	{
-		perror("sched_sighandler: write()");
-		pthread_mutex_unlock(&sched_master_lock);
-		exit(-1);
-	}
-
-	/*
-	 * Unlock the master mutex
-	 */
-	pthread_mutex_unlock(&sched_master_lock);
-}
-
-
-
-/*
- * ---------- sched_add_fdset
+/* ---------- 
+ * sched_add_fdset
  *
  * Add a file descriptor to one of the global scheduler sets and adjust
- * sched_numfd accordingly. ----------
+ * sched_numfd accordingly. 
+ * ----------
  */
 static void
 sched_add_fdset(int fd, fd_set * fds)
@@ -706,11 +687,12 @@ sched_add_fdset(int fd, fd_set * fds)
 }
 
 
-/*
- * ---------- sched_add_fdset
+/* ---------- 
+ * sched_add_fdset
  *
  * Remove a file descriptor from one of the global scheduler sets and adjust
- * sched_numfd accordingly. ----------
+ * sched_numfd accordingly. 
+ * ----------
  */
 static void
 sched_remove_fdset(int fd, fd_set * fds)
@@ -728,3 +710,5 @@ sched_remove_fdset(int fd, fd_set * fds)
 		}
 	}
 }
+
+
