@@ -3,10 +3,10 @@
  *
  *	Global definitions for the main replication engine.
  *
- *	Copyright (c) 2003-2006, PostgreSQL Global Development Group
+ *	Copyright (c) 2003-2004, PostgreSQL Global Development Group
  *	Author: Jan Wieck, Afilias USA INC.
  *
- *	$Id: slon.h,v 1.48.2.2 2006-10-25 12:57:07 wieck Exp $
+ *	$Id: slon.h,v 1.59 2006/03/08 18:29:10 darcyb Exp $
  *-------------------------------------------------------------------------
  */
 
@@ -37,7 +37,7 @@
 #else
 #define SLON_COMMANDS_PER_LINE		10
 #define SLON_DATA_FETCH_SIZE		10
-#define SLON_WORKLINES_PER_HELPER	(SLON_DATA_FETCH_SIZE * 50)
+#define SLON_WORKLINES_PER_HELPER	(SLON_DATA_FETCH_SIZE * 5)
 #endif
 
 #define SLON_MAX_PATH 1024
@@ -45,8 +45,8 @@
 /* See: http://archives.postgresql.org/pgsql-hackers/1999-10/msg00754.php */
 /* Also  view src/include/pg_config.h.win32 src/include/pg_config_manual.h */
 
-	/* FIXME: must determine and use OS specific max path length */
-	/* cbb: Not forcibly necessary; note that MAXPGPATH is 1024 */
+ /* FIXME: must determine and use OS specific max path length */
+ /* cbb: Not forcibly necessary; note that MAXPGPATH is 1024 */
 
 #define SLON_CLEANUP_SLEEP			600 /* sleep 10 minutes between */
  /* cleanup calls */
@@ -106,10 +106,6 @@ struct SlonNode_s
 	SlonWorkMsg *message_head;
 	SlonWorkMsg *message_tail;
 
-	char	   *archive_name;
-	char	   *archive_temp;
-	FILE	   *archive_fp;
-
 	SlonNode   *prev;
 	SlonNode   *next;
 };
@@ -159,6 +155,7 @@ struct SlonConn_s
 
 	int			condition;		/* what are we waiting for? */
 	struct timeval timeout;		/* timeofday for timeout */
+	int	pg_version;		/* PostgreSQL version */
 
 	SlonConn   *prev;
 	SlonConn   *next;
@@ -328,8 +325,12 @@ do { \
  * ----------
  */
 extern pid_t slon_pid;
-extern pid_t slon_ppid;
-extern pid_t slon_cpid;
+
+#ifndef WIN32
+extern pthread_mutex_t slon_watchdog_lock;
+extern pid_t slon_watchdog_pid;
+extern pid_t slon_worker_pid;
+#endif
 extern char *rtcfg_cluster_name;
 extern char *rtcfg_namespace;
 extern char *rtcfg_conninfo;
@@ -348,20 +349,64 @@ extern SlonSet *rtcfg_set_list_tail;
  * Functions in slon.c
  * ----------
  */
+#ifndef WIN32
 #define slon_abort() \
 do { \
-	kill((slon_ppid == 0 ? slon_pid : slon_ppid), SIGTERM); \
+	pthread_mutex_lock(&slon_watchdog_lock); \
+	if (slon_watchdog_pid >= 0) { \
+		slon_log(SLON_DEBUG2, "slon_abort() from pid=%d\n", slon_pid); \
+		kill(slon_watchdog_pid, SIGTERM); \
+		slon_watchdog_pid = -1; \
+	} \
+	pthread_mutex_unlock(&slon_watchdog_lock); \
 	pthread_exit(NULL); \
 } while (0)
 #define slon_restart() \
 do { \
-	kill((slon_ppid == 0 ? slon_pid : slon_ppid), SIGHUP); \
+	pthread_mutex_lock(&slon_watchdog_lock); \
+	if (slon_watchdog_pid >= 0) { \
+		slon_log(SLON_DEBUG2, "slon_restart() from pid=%d\n", slon_pid); \
+		kill(slon_watchdog_pid, SIGHUP); \
+		slon_watchdog_pid = -1; \
+	} \
+	pthread_mutex_unlock(&slon_watchdog_lock); \
+	pthread_exit(NULL); \
 } while (0)
-extern void slon_exit(int code);
+#define slon_retry() \
+do { \
+	pthread_mutex_lock(&slon_watchdog_lock); \
+	if (slon_watchdog_pid >= 0) { \
+		slon_log(SLON_DEBUG2, "slon_retry() from pid=%d\n", slon_pid); \
+		kill(slon_watchdog_pid, SIGUSR1); \
+		slon_watchdog_pid = -1; \
+	} \
+	pthread_mutex_unlock(&slon_watchdog_lock); \
+	pthread_exit(NULL); \
+} while (0)
+#else							/* WIN32 */
+/* On win32, we currently just bail out and let the service control manager
+ * deal with possible restarts */
+#define slon_abort() \
+do { \
+	WSACleanup(); \
+	exit(1); \
+} while (0)
+#define slon_restart() \
+do { \
+	WSACleanup(); \
+	exit(1); \
+} while (0)
+#define slon_retry() \
+do { \
+	WSACleanup(); \
+	exit(1); \
+} while (0)
+#endif
 
-extern int	slon_restart_request;
-extern int watchdog_pipe[];
-extern int sched_wakeuppipe[];
+extern void slon_exit(int code);
+extern void Usage(char *const argv[]);
+
+extern int	sched_wakeuppipe[];
 extern pthread_mutex_t slon_wait_listen_lock;
 extern pthread_cond_t slon_wait_listen_cond;
 
@@ -467,6 +512,8 @@ extern void *remoteListenThread_main(void *cdata);
  * ----------
  */
 extern int	sync_group_maxsize;
+extern int	sync_max_rowsize;
+extern int	sync_max_largemem;
 
 
 /* ----------
@@ -516,13 +563,24 @@ extern int	db_checkSchemaVersion(PGconn *conn);
 
 extern int	slon_mkquery(SlonDString * ds, char *fmt,...);
 extern int	slon_appendquery(SlonDString * ds, char *fmt,...);
-extern char 	*sql_on_connection;
+extern char *sql_on_connection;
 
 /* ----------
  * Globals in misc.c
  * ----------
  */
 extern int	slon_log_level;
+
+#if !defined(pgpipe) && !defined(WIN32)
+/* -----------------------------------
+ * pgpipe is not defined in PG pre-8.0
+ * -----------------------------------
+ */
+#define pgpipe(a)			pipe(a)
+#define piperead(a,b,c)		read(a,b,c)
+#define pipewrite(a,b,c)	write(a,b,c)
+#endif
+
 #endif   /* SLON_H_INCLUDED */
 
 
