@@ -3,17 +3,14 @@
  *
  *	Implementation of the thread that generates SYNC events.
  *
- *	Copyright (c) 2003-2006, PostgreSQL Global Development Group
+ *	Copyright (c) 2003-2004, PostgreSQL Global Development Group
  *	Author: Jan Wieck, Afilias USA INC.
  *
- *	$Id: sync_thread.c,v 1.15.2.1 2006-01-06 17:07:47 cbbrowne Exp $
+ *	$Id: sync_thread.c,v 1.17 2005/11/22 05:11:59 wieck Exp $
  *-------------------------------------------------------------------------
  */
 
 
-/* Note that in 1.1, generate_sync_event() is a stored procedure that
-   does roughly the same thing as this... */
-
 #include <pthread.h>
 
 #include <stdio.h>
@@ -31,15 +28,16 @@
 #include "slon.h"
 
 
-/*
- * ---------- Global variables ----------
+/* ---------- 
+ * Global variables 
+ * ----------
  */
 int			sync_interval;
 int			sync_interval_timeout;
 
 
-/*
- * ---------- slon_localSyncThread
+/* ---------- 
+ * slon_localSyncThread
  *
  * Generate SYNC event if local database activity created new log info.
  * ----------
@@ -62,7 +60,7 @@ syncThread_main(void *dummy)
 	 * Connect to the local database
 	 */
 	if ((conn = slon_connectdb(rtcfg_conninfo, "local_sync")) == NULL)
-		slon_abort();
+		slon_retry();
 	dbconn = conn->dbconn;
 
 	/*
@@ -106,7 +104,7 @@ syncThread_main(void *dummy)
 					 "syncThread: \"%s\" - %s",
 					 dstring_data(&query1), PQresultErrorMessage(res));
 			PQclear(res);
-			slon_abort();
+			slon_retry();
 			break;
 		}
 
@@ -134,7 +132,7 @@ syncThread_main(void *dummy)
 						 "syncThread: \"%s\" - %s",
 						 dstring_data(&query2), PQresultErrorMessage(res));
 				PQclear(res);
-				slon_abort();
+				slon_retry();
 				break;
 			}
 			slon_log(SLON_DEBUG2,
@@ -152,7 +150,7 @@ syncThread_main(void *dummy)
 						 "syncThread: \"commit transaction;\" - %s",
 						 PQresultErrorMessage(res));
 				PQclear(res);
-				slon_abort();
+				slon_retry();
 			}
 			PQclear(res);
 
@@ -175,7 +173,7 @@ syncThread_main(void *dummy)
 						 "syncThread: \"rollback transaction;\" - %s",
 						 PQresultErrorMessage(res));
 				PQclear(res);
-				slon_abort();
+				slon_retry();
 			}
 			PQclear(res);
 		}
