@@ -3,10 +3,10 @@
  *
  *	Definitions for slonik
  *
- *	Copyright (c) 2003-2006, PostgreSQL Global Development Group
+ *	Copyright (c) 2003-2004, PostgreSQL Global Development Group
  *	Author: Jan Wieck, Afilias USA INC.
  *
- *	$Id: slonik.h,v 1.23.2.3 2006-10-31 22:17:25 cbbrowne Exp $
+ *	$Id: slonik.h,v 1.27 2006/03/08 18:29:10 darcyb Exp $
  *-------------------------------------------------------------------------
  */
 
@@ -48,8 +48,7 @@ typedef struct SlonikStmt_move_set_s SlonikStmt_move_set;
 typedef struct SlonikStmt_ddl_script_s SlonikStmt_ddl_script;
 typedef struct SlonikStmt_update_functions_s SlonikStmt_update_functions;
 typedef struct SlonikStmt_wait_event_s SlonikStmt_wait_event;
-typedef struct SlonikStmt_sync_s SlonikStmt_sync;
-typedef struct SlonikStmt_sleep_s SlonikStmt_sleep;
+typedef struct SlonikStmt_switch_log_s SlonikStmt_switch_log;
 
 typedef enum
 {
@@ -87,9 +86,8 @@ typedef enum
 	STMT_UNSUBSCRIBE_SET,
 	STMT_UPDATE_FUNCTIONS,
 	STMT_WAIT_EVENT,
-	STMT_ERROR,
-	STMT_SYNC,
-	STMT_SLEEP
+	STMT_SWITCH_LOG,
+	STMT_ERROR
 }	Slonik_stmttype;
 
 struct SlonikScript_s
@@ -111,8 +109,7 @@ struct SlonikAdmInfo_s
 	char	   *conninfo;
 	PGconn	   *dbconn;
 	int64		last_event;
-	int			version_major;
-	int			version_minor;
+	int			pg_version;
 	int			nodeid_checked;
 	int			have_xact;
 	SlonikScript *script;
@@ -406,7 +403,7 @@ struct SlonikStmt_ddl_script_s
 	char	   *ddl_fname;
 	int			ev_origin;
 	int			only_on_node;
-	int			ddl_fd;
+	FILE	   *ddl_fd;
 };
 
 
@@ -427,18 +424,12 @@ struct SlonikStmt_wait_event_s
 };
 
 
-struct SlonikStmt_sync_s
+struct SlonikStmt_switch_log_s
 {
 	SlonikStmt	hdr;
 	int			no_id;
 };
 
-struct SlonikStmt_sleep_s
-{
-	SlonikStmt	hdr;
-	int			num_secs;
-};
-
 
 
 
@@ -561,8 +552,7 @@ extern int	slonik_move_set(SlonikStmt_move_set * stmt);
 extern int	slonik_ddl_script(SlonikStmt_ddl_script * stmt);
 extern int	slonik_update_functions(SlonikStmt_update_functions * stmt);
 extern int	slonik_wait_event(SlonikStmt_wait_event * stmt);
-extern int	slonik_sync(SlonikStmt_sync * stmt);
-extern int	slonik_sleep(SlonikStmt_sleep * stmt);
+extern int	slonik_switch_log(SlonikStmt_switch_log * stmt);
 
 extern int	slon_scanint64(char *str, int64 * result);
 
@@ -575,7 +565,6 @@ extern int	db_notice_lno;
 
 #ifdef HAVE_PQSETNOTICERECEIVER
 void		db_notice_recv(void *arg, const PGresult *res);
-
 #else
 void		db_notice_recv(void *arg, const char *msg);
 #endif
@@ -588,8 +577,7 @@ int db_exec_evcommand(SlonikStmt * stmt, SlonikAdmInfo * adminfo,
 				  SlonDString * query);
 PGresult *db_exec_select(SlonikStmt * stmt, SlonikAdmInfo * adminfo,
 			   SlonDString * query);
-int db_get_version(SlonikStmt * stmt, SlonikAdmInfo * adminfo,
-			   int *major, int *minor);
+int db_get_version(SlonikStmt * stmt, SlonikAdmInfo * adminfo);
 int db_check_namespace(SlonikStmt * stmt, SlonikAdmInfo * adminfo,
 				   char *clustername);
 int db_check_requirements(SlonikStmt * stmt, SlonikAdmInfo * adminfo,
